# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from .versions.client import VersionsClient
from .runs.client import RunsClient
from .indicators.client import IndicatorsClient
from ..core.request_options import RequestOptions
from ..types.prompt import Prompt
from ..core.pydantic_utilities import parse_obj_as
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..types.prompt_created_by import PromptCreatedBy
import datetime as dt
from ..types.prompt_organization import PromptOrganization
from ..types.prompt_associated_projects_item import PromptAssociatedProjectsItem
from ..core.serialization import convert_and_respect_annotation_metadata
from ..core.jsonable_encoder import jsonable_encoder
from .types.prompts_batch_predictions_request_results_item import PromptsBatchPredictionsRequestResultsItem
from .types.prompts_batch_predictions_response import PromptsBatchPredictionsResponse
from .types.prompts_batch_failed_predictions_request_failed_predictions_item import (
    PromptsBatchFailedPredictionsRequestFailedPredictionsItem,
)
from .types.prompts_batch_failed_predictions_response import PromptsBatchFailedPredictionsResponse
from ..core.client_wrapper import AsyncClientWrapper
from .versions.client import AsyncVersionsClient
from .runs.client import AsyncRunsClient
from .indicators.client import AsyncIndicatorsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class PromptsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper
        self.versions = VersionsClient(client_wrapper=self._client_wrapper)
        self.runs = RunsClient(client_wrapper=self._client_wrapper)
        self.indicators = IndicatorsClient(client_wrapper=self._client_wrapper)

    def list(self, *, request_options: typing.Optional[RequestOptions] = None) -> typing.List[Prompt]:
        """
        Get a list of prompts.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Prompt]


        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.prompts.list()
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/prompts/",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[Prompt],
                    parse_obj_as(
                        type_=typing.List[Prompt],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        *,
        title: str,
        input_fields: typing.Sequence[str],
        output_classes: typing.Sequence[str],
        description: typing.Optional[str] = OMIT,
        created_by: typing.Optional[PromptCreatedBy] = OMIT,
        created_at: typing.Optional[dt.datetime] = OMIT,
        updated_at: typing.Optional[dt.datetime] = OMIT,
        organization: typing.Optional[PromptOrganization] = OMIT,
        associated_projects: typing.Optional[typing.Sequence[PromptAssociatedProjectsItem]] = OMIT,
        skill_name: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Prompt:
        """
        Create a new prompt.

        Parameters
        ----------
        title : str
            Title of the prompt

        input_fields : typing.Sequence[str]
            List of input fields

        output_classes : typing.Sequence[str]
            List of output classes

        description : typing.Optional[str]
            Description of the prompt

        created_by : typing.Optional[PromptCreatedBy]
            User ID of the creator of the prompt

        created_at : typing.Optional[dt.datetime]
            Date and time the prompt was created

        updated_at : typing.Optional[dt.datetime]
            Date and time the prompt was last updated

        organization : typing.Optional[PromptOrganization]
            Organization ID of the prompt

        associated_projects : typing.Optional[typing.Sequence[PromptAssociatedProjectsItem]]
            List of associated projects IDs or objects

        skill_name : typing.Optional[str]
            Name of the skill

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Prompt


        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.prompts.create(
            title="title",
            input_fields=["input_fields"],
            output_classes=["output_classes"],
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/prompts/",
            method="POST",
            json={
                "title": title,
                "description": description,
                "created_by": convert_and_respect_annotation_metadata(
                    object_=created_by, annotation=PromptCreatedBy, direction="write"
                ),
                "created_at": created_at,
                "updated_at": updated_at,
                "organization": convert_and_respect_annotation_metadata(
                    object_=organization, annotation=PromptOrganization, direction="write"
                ),
                "input_fields": input_fields,
                "output_classes": output_classes,
                "associated_projects": convert_and_respect_annotation_metadata(
                    object_=associated_projects,
                    annotation=typing.Sequence[PromptAssociatedProjectsItem],
                    direction="write",
                ),
                "skill_name": skill_name,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Prompt,
                    parse_obj_as(
                        type_=Prompt,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(self, id: int, *, request_options: typing.Optional[RequestOptions] = None) -> Prompt:
        """
        Get a prompt by ID.

        Parameters
        ----------
        id : int
            Prompt ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Prompt


        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.prompts.get(
            id=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/prompts/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Prompt,
                    parse_obj_as(
                        type_=Prompt,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(self, id: int, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a prompt by ID.

        Parameters
        ----------
        id : int
            Prompt ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.prompts.delete(
            id=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/prompts/{jsonable_encoder(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self,
        id: int,
        *,
        title: str,
        input_fields: typing.Sequence[str],
        output_classes: typing.Sequence[str],
        description: typing.Optional[str] = OMIT,
        created_by: typing.Optional[PromptCreatedBy] = OMIT,
        created_at: typing.Optional[dt.datetime] = OMIT,
        updated_at: typing.Optional[dt.datetime] = OMIT,
        organization: typing.Optional[PromptOrganization] = OMIT,
        associated_projects: typing.Optional[typing.Sequence[PromptAssociatedProjectsItem]] = OMIT,
        skill_name: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Prompt:
        """
        Update a prompt by ID.

        Parameters
        ----------
        id : int
            Prompt ID

        title : str
            Title of the prompt

        input_fields : typing.Sequence[str]
            List of input fields

        output_classes : typing.Sequence[str]
            List of output classes

        description : typing.Optional[str]
            Description of the prompt

        created_by : typing.Optional[PromptCreatedBy]
            User ID of the creator of the prompt

        created_at : typing.Optional[dt.datetime]
            Date and time the prompt was created

        updated_at : typing.Optional[dt.datetime]
            Date and time the prompt was last updated

        organization : typing.Optional[PromptOrganization]
            Organization ID of the prompt

        associated_projects : typing.Optional[typing.Sequence[PromptAssociatedProjectsItem]]
            List of associated projects IDs or objects

        skill_name : typing.Optional[str]
            Name of the skill

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Prompt


        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.prompts.update(
            id=1,
            title="title",
            input_fields=["input_fields"],
            output_classes=["output_classes"],
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/prompts/{jsonable_encoder(id)}",
            method="PATCH",
            json={
                "title": title,
                "description": description,
                "created_by": convert_and_respect_annotation_metadata(
                    object_=created_by, annotation=PromptCreatedBy, direction="write"
                ),
                "created_at": created_at,
                "updated_at": updated_at,
                "organization": convert_and_respect_annotation_metadata(
                    object_=organization, annotation=PromptOrganization, direction="write"
                ),
                "input_fields": input_fields,
                "output_classes": output_classes,
                "associated_projects": convert_and_respect_annotation_metadata(
                    object_=associated_projects,
                    annotation=typing.Sequence[PromptAssociatedProjectsItem],
                    direction="write",
                ),
                "skill_name": skill_name,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Prompt,
                    parse_obj_as(
                        type_=Prompt,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def batch_predictions(
        self,
        *,
        num_predictions: typing.Optional[int] = None,
        modelrun_id: typing.Optional[int] = OMIT,
        results: typing.Optional[typing.Sequence[PromptsBatchPredictionsRequestResultsItem]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PromptsBatchPredictionsResponse:
        """
        Create a new batch prediction.

        Parameters
        ----------
        num_predictions : typing.Optional[int]
            Number of predictions being sent

        modelrun_id : typing.Optional[int]
            Model Run ID to associate the prediction with

        results : typing.Optional[typing.Sequence[PromptsBatchPredictionsRequestResultsItem]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PromptsBatchPredictionsResponse


        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.prompts.batch_predictions()
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/model-run/batch-predictions",
            method="POST",
            params={
                "num_predictions": num_predictions,
            },
            json={
                "modelrun_id": modelrun_id,
                "results": convert_and_respect_annotation_metadata(
                    object_=results,
                    annotation=typing.Sequence[PromptsBatchPredictionsRequestResultsItem],
                    direction="write",
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    PromptsBatchPredictionsResponse,
                    parse_obj_as(
                        type_=PromptsBatchPredictionsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def batch_failed_predictions(
        self,
        *,
        num_failed_predictions: typing.Optional[int] = None,
        modelrun_id: typing.Optional[int] = OMIT,
        failed_predictions: typing.Optional[
            typing.Sequence[PromptsBatchFailedPredictionsRequestFailedPredictionsItem]
        ] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PromptsBatchFailedPredictionsResponse:
        """
        Create a new batch of failed predictions.

        Parameters
        ----------
        num_failed_predictions : typing.Optional[int]
            Number of failed predictions being sent

        modelrun_id : typing.Optional[int]
            Model Run ID where the failed predictions came from

        failed_predictions : typing.Optional[typing.Sequence[PromptsBatchFailedPredictionsRequestFailedPredictionsItem]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PromptsBatchFailedPredictionsResponse


        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.prompts.batch_failed_predictions()
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/model-run/batch-failed-predictions",
            method="POST",
            params={
                "num_failed_predictions": num_failed_predictions,
            },
            json={
                "modelrun_id": modelrun_id,
                "failed_predictions": convert_and_respect_annotation_metadata(
                    object_=failed_predictions,
                    annotation=typing.Sequence[PromptsBatchFailedPredictionsRequestFailedPredictionsItem],
                    direction="write",
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    PromptsBatchFailedPredictionsResponse,
                    parse_obj_as(
                        type_=PromptsBatchFailedPredictionsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncPromptsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper
        self.versions = AsyncVersionsClient(client_wrapper=self._client_wrapper)
        self.runs = AsyncRunsClient(client_wrapper=self._client_wrapper)
        self.indicators = AsyncIndicatorsClient(client_wrapper=self._client_wrapper)

    async def list(self, *, request_options: typing.Optional[RequestOptions] = None) -> typing.List[Prompt]:
        """
        Get a list of prompts.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Prompt]


        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.prompts.list()


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/prompts/",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[Prompt],
                    parse_obj_as(
                        type_=typing.List[Prompt],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        *,
        title: str,
        input_fields: typing.Sequence[str],
        output_classes: typing.Sequence[str],
        description: typing.Optional[str] = OMIT,
        created_by: typing.Optional[PromptCreatedBy] = OMIT,
        created_at: typing.Optional[dt.datetime] = OMIT,
        updated_at: typing.Optional[dt.datetime] = OMIT,
        organization: typing.Optional[PromptOrganization] = OMIT,
        associated_projects: typing.Optional[typing.Sequence[PromptAssociatedProjectsItem]] = OMIT,
        skill_name: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Prompt:
        """
        Create a new prompt.

        Parameters
        ----------
        title : str
            Title of the prompt

        input_fields : typing.Sequence[str]
            List of input fields

        output_classes : typing.Sequence[str]
            List of output classes

        description : typing.Optional[str]
            Description of the prompt

        created_by : typing.Optional[PromptCreatedBy]
            User ID of the creator of the prompt

        created_at : typing.Optional[dt.datetime]
            Date and time the prompt was created

        updated_at : typing.Optional[dt.datetime]
            Date and time the prompt was last updated

        organization : typing.Optional[PromptOrganization]
            Organization ID of the prompt

        associated_projects : typing.Optional[typing.Sequence[PromptAssociatedProjectsItem]]
            List of associated projects IDs or objects

        skill_name : typing.Optional[str]
            Name of the skill

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Prompt


        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.prompts.create(
                title="title",
                input_fields=["input_fields"],
                output_classes=["output_classes"],
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/prompts/",
            method="POST",
            json={
                "title": title,
                "description": description,
                "created_by": convert_and_respect_annotation_metadata(
                    object_=created_by, annotation=PromptCreatedBy, direction="write"
                ),
                "created_at": created_at,
                "updated_at": updated_at,
                "organization": convert_and_respect_annotation_metadata(
                    object_=organization, annotation=PromptOrganization, direction="write"
                ),
                "input_fields": input_fields,
                "output_classes": output_classes,
                "associated_projects": convert_and_respect_annotation_metadata(
                    object_=associated_projects,
                    annotation=typing.Sequence[PromptAssociatedProjectsItem],
                    direction="write",
                ),
                "skill_name": skill_name,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Prompt,
                    parse_obj_as(
                        type_=Prompt,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(self, id: int, *, request_options: typing.Optional[RequestOptions] = None) -> Prompt:
        """
        Get a prompt by ID.

        Parameters
        ----------
        id : int
            Prompt ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Prompt


        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.prompts.get(
                id=1,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/prompts/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Prompt,
                    parse_obj_as(
                        type_=Prompt,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(self, id: int, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a prompt by ID.

        Parameters
        ----------
        id : int
            Prompt ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.prompts.delete(
                id=1,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/prompts/{jsonable_encoder(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self,
        id: int,
        *,
        title: str,
        input_fields: typing.Sequence[str],
        output_classes: typing.Sequence[str],
        description: typing.Optional[str] = OMIT,
        created_by: typing.Optional[PromptCreatedBy] = OMIT,
        created_at: typing.Optional[dt.datetime] = OMIT,
        updated_at: typing.Optional[dt.datetime] = OMIT,
        organization: typing.Optional[PromptOrganization] = OMIT,
        associated_projects: typing.Optional[typing.Sequence[PromptAssociatedProjectsItem]] = OMIT,
        skill_name: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Prompt:
        """
        Update a prompt by ID.

        Parameters
        ----------
        id : int
            Prompt ID

        title : str
            Title of the prompt

        input_fields : typing.Sequence[str]
            List of input fields

        output_classes : typing.Sequence[str]
            List of output classes

        description : typing.Optional[str]
            Description of the prompt

        created_by : typing.Optional[PromptCreatedBy]
            User ID of the creator of the prompt

        created_at : typing.Optional[dt.datetime]
            Date and time the prompt was created

        updated_at : typing.Optional[dt.datetime]
            Date and time the prompt was last updated

        organization : typing.Optional[PromptOrganization]
            Organization ID of the prompt

        associated_projects : typing.Optional[typing.Sequence[PromptAssociatedProjectsItem]]
            List of associated projects IDs or objects

        skill_name : typing.Optional[str]
            Name of the skill

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Prompt


        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.prompts.update(
                id=1,
                title="title",
                input_fields=["input_fields"],
                output_classes=["output_classes"],
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/prompts/{jsonable_encoder(id)}",
            method="PATCH",
            json={
                "title": title,
                "description": description,
                "created_by": convert_and_respect_annotation_metadata(
                    object_=created_by, annotation=PromptCreatedBy, direction="write"
                ),
                "created_at": created_at,
                "updated_at": updated_at,
                "organization": convert_and_respect_annotation_metadata(
                    object_=organization, annotation=PromptOrganization, direction="write"
                ),
                "input_fields": input_fields,
                "output_classes": output_classes,
                "associated_projects": convert_and_respect_annotation_metadata(
                    object_=associated_projects,
                    annotation=typing.Sequence[PromptAssociatedProjectsItem],
                    direction="write",
                ),
                "skill_name": skill_name,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Prompt,
                    parse_obj_as(
                        type_=Prompt,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def batch_predictions(
        self,
        *,
        num_predictions: typing.Optional[int] = None,
        modelrun_id: typing.Optional[int] = OMIT,
        results: typing.Optional[typing.Sequence[PromptsBatchPredictionsRequestResultsItem]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PromptsBatchPredictionsResponse:
        """
        Create a new batch prediction.

        Parameters
        ----------
        num_predictions : typing.Optional[int]
            Number of predictions being sent

        modelrun_id : typing.Optional[int]
            Model Run ID to associate the prediction with

        results : typing.Optional[typing.Sequence[PromptsBatchPredictionsRequestResultsItem]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PromptsBatchPredictionsResponse


        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.prompts.batch_predictions()


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/model-run/batch-predictions",
            method="POST",
            params={
                "num_predictions": num_predictions,
            },
            json={
                "modelrun_id": modelrun_id,
                "results": convert_and_respect_annotation_metadata(
                    object_=results,
                    annotation=typing.Sequence[PromptsBatchPredictionsRequestResultsItem],
                    direction="write",
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    PromptsBatchPredictionsResponse,
                    parse_obj_as(
                        type_=PromptsBatchPredictionsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def batch_failed_predictions(
        self,
        *,
        num_failed_predictions: typing.Optional[int] = None,
        modelrun_id: typing.Optional[int] = OMIT,
        failed_predictions: typing.Optional[
            typing.Sequence[PromptsBatchFailedPredictionsRequestFailedPredictionsItem]
        ] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PromptsBatchFailedPredictionsResponse:
        """
        Create a new batch of failed predictions.

        Parameters
        ----------
        num_failed_predictions : typing.Optional[int]
            Number of failed predictions being sent

        modelrun_id : typing.Optional[int]
            Model Run ID where the failed predictions came from

        failed_predictions : typing.Optional[typing.Sequence[PromptsBatchFailedPredictionsRequestFailedPredictionsItem]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PromptsBatchFailedPredictionsResponse


        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.prompts.batch_failed_predictions()


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/model-run/batch-failed-predictions",
            method="POST",
            params={
                "num_failed_predictions": num_failed_predictions,
            },
            json={
                "modelrun_id": modelrun_id,
                "failed_predictions": convert_and_respect_annotation_metadata(
                    object_=failed_predictions,
                    annotation=typing.Sequence[PromptsBatchFailedPredictionsRequestFailedPredictionsItem],
                    direction="write",
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    PromptsBatchFailedPredictionsResponse,
                    parse_obj_as(
                        type_=PromptsBatchFailedPredictionsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
from .comment_created_by import CommentCreatedBy
import datetime as dt
import typing
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class Comment(UniversalBaseModel):
    id: int
    text: str
    project: int
    task: int
    annotation: int
    created_by: CommentCreatedBy
    created_at: dt.datetime
    updated_at: dt.datetime
    is_resolved: typing.Optional[bool] = None
    resolved_at: typing.Optional[dt.datetime] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
from .ml_create_response_auth_method import MlCreateResponseAuthMethod
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class MlCreateResponse(UniversalBaseModel):
    url: typing.Optional[str] = pydantic.Field(default=None)
    """
    ML backend URL
    """

    project: typing.Optional[int] = pydantic.Field(default=None)
    """
    Project ID
    """

    is_interactive: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Is interactive
    """

    title: typing.Optional[str] = pydantic.Field(default=None)
    """
    Title
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Description
    """

    auth_method: typing.Optional[MlCreateResponseAuthMethod] = pydantic.Field(default=None)
    """
    Auth method
    """

    basic_auth_user: typing.Optional[str] = pydantic.Field(default=None)
    """
    Basic auth user
    """

    basic_auth_pass: typing.Optional[str] = pydantic.Field(default=None)
    """
    Basic auth password
    """

    extra_params: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = pydantic.Field(default=None)
    """
    Extra parameters
    """

    timeout: typing.Optional[int] = pydantic.Field(default=None)
    """
    Response model timeout
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .annotations_create_bulk_request_selected_items import AnnotationsCreateBulkRequestSelectedItems
from .annotations_create_bulk_response_item import AnnotationsCreateBulkResponseItem

__all__ = ["AnnotationsCreateBulkRequestSelectedItems", "AnnotationsCreateBulkResponseItem"]



# This file was auto-generated by Fern from our API Definition.

from ....core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
from ....core.pydantic_utilities import IS_PYDANTIC_V2


class GcsUpdateResponse(UniversalBaseModel):
    regex_filter: typing.Optional[str] = pydantic.Field(default=None)
    """
    Cloud storage regex for filtering objects. You must specify it otherwise no objects will be imported.
    """

    use_blob_urls: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Interpret objects as BLOBs and generate URLs. For example, if your bucket contains images, you can use this option to generate URLs for these images. If set to False, it will read the content of the file and load it into Label Studio.
    """

    presign: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Presign URLs for direct download
    """

    presign_ttl: typing.Optional[int] = pydantic.Field(default=None)
    """
    Presign TTL in minutes
    """

    title: typing.Optional[str] = pydantic.Field(default=None)
    """
    Storage title
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Storage description
    """

    project: typing.Optional[int] = pydantic.Field(default=None)
    """
    Project ID
    """

    bucket: typing.Optional[str] = pydantic.Field(default=None)
    """
    GCS bucket name
    """

    prefix: typing.Optional[str] = pydantic.Field(default=None)
    """
    GCS bucket prefix
    """

    google_application_credentials: typing.Optional[str] = pydantic.Field(default=None)
    """
    The content of GOOGLE_APPLICATION_CREDENTIALS json file. Check official Google Cloud Authentication documentation for more details.
    """

    google_project_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Google project ID
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing
from .views_create_request_data_filters import ViewsCreateRequestDataFilters
import pydantic
from .views_create_request_data_ordering_item import ViewsCreateRequestDataOrderingItem
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class ViewsCreateRequestData(UniversalBaseModel):
    """
    Custom view data
    """

    filters: typing.Optional[ViewsCreateRequestDataFilters] = pydantic.Field(default=None)
    """
    Filters to apply on tasks. You can use [the helper class `Filters` from this page](https://labelstud.io/sdk/data_manager.html) to create Data Manager Filters.<br>Example: `{"conjunction": "or", "items": [{"filter": "filter:tasks:completed_at", "operator": "greater", "type": "Datetime", "value": "2021-01-01T00:00:00.000Z"}]}`
    """

    ordering: typing.Optional[typing.List[ViewsCreateRequestDataOrderingItem]] = pydantic.Field(default=None)
    """
    List of fields to order by. Fields are similar to filters but without the `filter:` prefix. To reverse the order, add a minus sign before the field name, e.g. `-tasks:created_at`.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
from .inference_run_organization import InferenceRunOrganization
from .inference_run_created_by import InferenceRunCreatedBy
from .inference_run_project_subset import InferenceRunProjectSubset
from .inference_run_status import InferenceRunStatus
import datetime as dt
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class InferenceRun(UniversalBaseModel):
    organization: typing.Optional[InferenceRunOrganization] = None
    project: int
    model_version: typing.Optional[int] = None
    created_by: typing.Optional[InferenceRunCreatedBy] = None
    project_subset: InferenceRunProjectSubset
    status: typing.Optional[InferenceRunStatus] = None
    job_id: typing.Optional[str] = None
    created_at: typing.Optional[dt.datetime] = None
    triggered_at: typing.Optional[dt.datetime] = None
    predictions_updated_at: typing.Optional[dt.datetime] = None
    completed_at: typing.Optional[dt.datetime] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
from .prompt_version_provider import PromptVersionProvider
from .prompt_version_created_by import PromptVersionCreatedBy
import datetime as dt
from .prompt_version_organization import PromptVersionOrganization
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class PromptVersion(UniversalBaseModel):
    title: typing.Optional[str] = None
    parent_model: typing.Optional[int] = None
    model_provider_connection: typing.Optional[int] = None
    prompt: typing.Optional[str] = None
    provider: typing.Optional[PromptVersionProvider] = None
    provider_model_id: typing.Optional[str] = None
    created_by: typing.Optional[PromptVersionCreatedBy] = None
    created_at: typing.Optional[dt.datetime] = None
    updated_at: typing.Optional[dt.datetime] = None
    organization: typing.Optional[PromptVersionOrganization] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
import datetime as dt
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class BaseUser(UniversalBaseModel):
    id: typing.Optional[int] = None
    first_name: typing.Optional[str] = None
    last_name: typing.Optional[str] = None
    username: str
    email: typing.Optional[str] = None
    last_activity: typing.Optional[dt.datetime] = None
    avatar: typing.Optional[str] = None
    initials: typing.Optional[str] = None
    phone: typing.Optional[str] = None
    active_organization: typing.Optional[int] = None
    allow_newsletters: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Allow sending newsletters to user
    """

    date_joined: typing.Optional[dt.datetime] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
import datetime as dt
import typing_extensions
from ..core.serialization import FieldMetadata
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class S3SExportStorage(UniversalBaseModel):
    id: typing.Optional[int] = None
    title: typing.Optional[str] = pydantic.Field(default=None)
    """
    Cloud storage title
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Cloud storage description
    """

    created_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Creation time
    """

    bucket: typing.Optional[str] = pydantic.Field(default=None)
    """
    S3 bucket name
    """

    prefix: typing.Optional[str] = pydantic.Field(default=None)
    """
    S3 bucket prefix
    """

    external_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    AWS External ID
    """

    role_arn: typing.Optional[str] = pydantic.Field(default=None)
    """
    AWS Role ARN
    """

    region_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    AWS Region
    """

    s3endpoint: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="s3_endpoint")] = pydantic.Field(
        default=None
    )
    """
    S3 Endpoint
    """

    project: int = pydantic.Field()
    """
    A unique integer value identifying this project.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ....core.pydantic_utilities import UniversalBaseModel
import typing
from ....types.export_format import ExportFormat
import pydantic
from ....core.pydantic_utilities import IS_PYDANTIC_V2


class ExportsConvertResponse(UniversalBaseModel):
    export_type: typing.Optional[ExportFormat] = None
    converted_format: typing.Optional[int] = pydantic.Field(default=None)
    """
    ID of the converted format
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class InferenceRunCostEstimate(UniversalBaseModel):
    prompt_cost_usd: typing.Optional[str] = pydantic.Field(default=None)
    """
    Cost of the prompt (in USD)
    """

    completion_cost_usd: typing.Optional[str] = pydantic.Field(default=None)
    """
    Cost of the completion (in USD)
    """

    total_cost_usd: typing.Optional[str] = pydantic.Field(default=None)
    """
    Total cost of the inference (in USD)
    """

    is_error: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether an error occurred or not
    """

    error_type: typing.Optional[str] = pydantic.Field(default=None)
    """
    Type of error (e.g. "Timeout", "Rate Limit", etc)
    """

    error_message: typing.Optional[str] = pydantic.Field(default=None)
    """
    Error message details
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .types import (
    ProjectsCreateResponse,
    ProjectsImportTasksResponse,
    ProjectsListRequestFilter,
    ProjectsListResponse,
    ProjectsUpdateResponse,
)
from . import exports, pauses
from .exports import ExportsConvertResponse, ExportsListFormatsResponseItem

__all__ = [
    "ExportsConvertResponse",
    "ExportsListFormatsResponseItem",
    "ProjectsCreateResponse",
    "ProjectsImportTasksResponse",
    "ProjectsListRequestFilter",
    "ProjectsListResponse",
    "ProjectsUpdateResponse",
    "exports",
    "pauses",
]
